library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity a2gx_config is 
	port (
		clkin_bot_p : in std_logic;
		cpu_resetn : in std_logic;

		enet_gtx_clk : out std_logic;      --2.5V  --RGMII Transmit Clock
		enet_tx_d : out std_logic_vector(3 downto 0);        --2.5V  --TX to PHY
		enet_rx_d : in std_logic_vector(3 downto 0);        --2.5V  --RX from PHY
		enet_tx_en : out std_logic;       --2.5V  --RGMII Transmit Control
		enet_rx_clk : in std_logic;      --2.5V  --Derived Received Clock
		enet_rx_dv : in std_logic;       --2.5V  --RGMII Receive Control 
		enet_resetn : out std_logic;        --2.5V      --Reset to PHY (TR=0)
		enet_mdc : out std_logic;           --2.5V      --MDIO Control (TR=0)
		enet_mdio : inout std_logic;          --2.5V      --MDIO Data (TR=0)
		enet_intn : in std_logic;           --2.5V      --MDIO Interrupt (TR=0)

		fsm_a : out std_logic_vector(25 downto 0);              --2.5V      --FSM Address Bus (1Gb Flash)
		fsm_d : inout std_logic_vector(31 downto 0);              --2.5V      --FSM Data Bus
		flash_clk : out std_logic;          --2.5V  
		flash_cen : out std_logic;          --2.5V  
		flash_oen : out std_logic;          --2.5V
		flash_wen : out std_logic;          --2.5V
		flash_advn : out std_logic;         --2.5V
		flash_rdybsyn : in std_logic;      --2.5V
		flash_resetn : out std_logic;       --2.5V     -- (TR=0)
		sram_clk : out std_logic;           --2.5V
		sram_cen : out std_logic;           --2.5V
		sram_dqp : inout std_logic_vector(3 downto 0);           --2.5V     --Parity bits only go to SRAM
		sram_bwn : out std_logic_vector(3 downto 0);           --2.5V
		sram_gwn : out std_logic;           --2.5V
		sram_bwen : out std_logic;          --2.5V
		sram_oen : out std_logic;           --2.5V
		--sram_advn,          --2.5V
		--sram_adspn,         --2.5V
		sram_adscn : out std_logic;         --2.5V
		sram_zz : out std_logic;            --2.5V     -- (TR=0)
		max2_clk : out std_logic;           --1.8V
		max2_csn : out std_logic;           --1.8V
		max2_ben : out std_logic_vector(3 downto 0);           --1.8V
		max2_oen : out std_logic;           --1.8V
		max2_wen : out std_logic;           --1.8V

		lcd_data : inout std_logic_vector(7 downto 0);
		lcd_d_cn : out std_logic;
		lcd_wen : out std_logic;
		lcd_csn : out std_logic;

		user_dipsw : in std_logic_vector(3 downto 0);
		user_led : out std_logic_vector(3 downto 0);
		user_pb : in std_logic_vector(1 downto 0);
		
		pcie_led_x1 : out std_logic;
		pcie_led_x4 : out std_logic;
		pcie_led_x8 : out std_logic
		
	);
end entity a2gx_config;


architecture rtl of a2gx_config is
	
component bup_nios is 
           port (
                 -- 1) global signals:
                    signal clk_0 : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal sram_clk : OUT STD_LOGIC;
                    signal sys_clk : OUT STD_LOGIC;

                 -- the_dip_pio
                    signal in_port_to_the_dip_pio : IN STD_LOGIC_VECTOR (3 DOWNTO 0);

                 -- the_lcd
                    signal LCD_E_from_the_lcd : OUT STD_LOGIC;
                    signal LCD_RS_from_the_lcd : OUT STD_LOGIC;
                    signal LCD_RW_from_the_lcd : OUT STD_LOGIC;
                    signal LCD_data_to_and_from_the_lcd : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);

                 -- the_led_pio
                    signal out_port_from_the_led_pio : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);

                 -- the_pb_pio
                    signal in_port_to_the_pb_pio : IN STD_LOGIC_VECTOR (1 DOWNTO 0);

                 -- the_tri_state_bridge_0_avalon_slave
                    signal adsc_n_to_the_max2_inf : OUT STD_LOGIC;
                    signal adsc_n_to_the_ssram : OUT STD_LOGIC;
                    signal bw_n_to_the_max2_inf : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
                    signal bw_n_to_the_ssram : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
                    signal bwe_n_to_the_max2_inf : OUT STD_LOGIC;
                    signal bwe_n_to_the_ssram : OUT STD_LOGIC;
                    signal chipenable1_n_to_the_max2_inf : OUT STD_LOGIC;
                    signal chipenable1_n_to_the_ssram : OUT STD_LOGIC;
                    signal outputenable_n_to_the_max2_inf : OUT STD_LOGIC;
                    signal outputenable_n_to_the_ssram : OUT STD_LOGIC;
                    signal read_n_to_the_ext_flash : OUT STD_LOGIC;
                    signal read_n_to_the_ext_flash_1 : OUT STD_LOGIC;
                    signal select_n_to_the_ext_flash : OUT STD_LOGIC;
                    signal select_n_to_the_ext_flash_1 : OUT STD_LOGIC;
                    signal flash_tristate_bridge_address : OUT STD_LOGIC_VECTOR (24 DOWNTO 0);
                    signal flash_tristate_bridge_data : INOUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                    signal write_n_to_the_ext_flash : OUT STD_LOGIC;
                    signal write_n_to_the_ext_flash_1 : OUT STD_LOGIC;
                    
                 -- the_tse_mac
                    signal ena_10_from_the_tse_mac : OUT STD_LOGIC;
                    signal eth_mode_from_the_tse_mac : OUT STD_LOGIC;
                    signal mdc_from_the_tse_mac : OUT STD_LOGIC;
                    signal mdio_in_to_the_tse_mac : IN STD_LOGIC;
                    signal mdio_oen_from_the_tse_mac : OUT STD_LOGIC;
                    signal mdio_out_from_the_tse_mac : OUT STD_LOGIC;
                    signal rgmii_in_to_the_tse_mac : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
                    signal rgmii_out_from_the_tse_mac : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
                    signal rx_clk_to_the_tse_mac : IN STD_LOGIC;
                    signal rx_control_to_the_tse_mac : IN STD_LOGIC;
                    signal set_1000_to_the_tse_mac : IN STD_LOGIC;
                    signal set_10_to_the_tse_mac : IN STD_LOGIC;
                    signal tx_clk_to_the_tse_mac : IN STD_LOGIC;
                    signal tx_control_from_the_tse_mac : OUT STD_LOGIC
                    
                 );
	end component;

	component pll_125 is 
		port (
			inclk0	: IN STD_LOGIC  := '0';
			c0		: OUT STD_LOGIC ;
			c1		: OUT STD_LOGIC ;
			c2		: OUT STD_LOGIC 
			
		);
	END component;

	component enet_gtx_clk_ddio_buffer IS
		PORT(
		aclr		: IN STD_LOGIC  := '0';
		datain_h		: IN STD_LOGIC ;
		datain_l		: IN STD_LOGIC ;
		outclock		: IN STD_LOGIC ;
		dataout		: OUT STD_LOGIC 
		);
	END component;


	signal max2_adsc_int, max2_cs_int : std_logic;
	signal flash_oe, flash_1_oe, flash_ce, flash_1_ce, flash_we, flash_1_we : std_logic;

	signal mdio_in_to_the_tse_mac : STD_LOGIC;
	signal mdio_oen_from_the_tse_mac : STD_LOGIC;
	signal mdio_out_from_the_tse_mac : STD_LOGIC;
	signal ena_10_from_the_tse_mac : STD_LOGIC;
	signal eth_mode_from_the_tse_mac : STD_LOGIC;

	signal pll_clk_125, pll_clk_25, pll_clk_2 : std_logic;
	signal tx_clk_to_the_tse_mac : std_logic;

	signal count : integer range 0 to 65535;
	signal reset_int : std_logic;
	signal sram_clk_int, dummy : std_logic;
begin

  bup_nios_inst : bup_nios
    port map(
    
      reset_n => cpu_resetn,
      clk_0 => clkin_bot_p,
      sram_clk => sram_clk_int,
      sys_clk => dummy,


      LCD_E_from_the_lcd => lcd_csn,
      LCD_RS_from_the_lcd => lcd_d_cn,
      LCD_RW_from_the_lcd => lcd_wen,
      LCD_data_to_and_from_the_lcd => lcd_data,

   
      adsc_n_to_the_max2_inf => max2_adsc_int,
      bw_n_to_the_max2_inf => max2_ben,
      bwe_n_to_the_max2_inf => max2_wen,
      chipenable1_n_to_the_max2_inf => max2_csn,
      outputenable_n_to_the_max2_inf => max2_oen,

      adsc_n_to_the_ssram => sram_adscn,
      bw_n_to_the_ssram => sram_bwn,
      bwe_n_to_the_ssram => sram_bwen,
      chipenable1_n_to_the_ssram => sram_cen,
      outputenable_n_to_the_ssram => sram_oen,

      flash_tristate_bridge_address => fsm_a(24 downto 0),
      flash_tristate_bridge_data => fsm_d,
      
      read_n_to_the_ext_flash => 		flash_oe,
      read_n_to_the_ext_flash_1 => 		flash_1_oe,
      select_n_to_the_ext_flash => 		flash_ce,
      select_n_to_the_ext_flash_1 => 	flash_1_ce,
      write_n_to_the_ext_flash => 		flash_we,
      write_n_to_the_ext_flash_1 => 	flash_1_we,

      in_port_to_the_dip_pio => user_dipsw,
      in_port_to_the_pb_pio => user_pb,
      out_port_from_the_led_pio => user_led,
      
		ena_10_from_the_tse_mac => ena_10_from_the_tse_mac,
		eth_mode_from_the_tse_mac => eth_mode_from_the_tse_mac,

		mdc_from_the_tse_mac => enet_mdc,
		mdio_in_to_the_tse_mac => mdio_in_to_the_tse_mac,
		mdio_oen_from_the_tse_mac => mdio_oen_from_the_tse_mac,
		mdio_out_from_the_tse_mac => mdio_out_from_the_tse_mac,
		rgmii_in_to_the_tse_mac => enet_rx_d,
		rgmii_out_from_the_tse_mac => enet_tx_d,
		rx_clk_to_the_tse_mac => enet_rx_clk,
		rx_control_to_the_tse_mac => enet_rx_dv,
		tx_control_from_the_tse_mac => enet_tx_en,

		set_1000_to_the_tse_mac => 'Z',
		set_10_to_the_tse_mac => 'Z',
		tx_clk_to_the_tse_mac =>tx_clk_to_the_tse_mac




    );

	process(mdio_oen_from_the_tse_mac, mdio_out_from_the_tse_mac)begin
		if(mdio_oen_from_the_tse_mac = '0')then
			enet_mdio <= mdio_out_from_the_tse_mac;
		else
			enet_mdio <= 'Z';  
		end if;
	end process;


	
	process(ena_10_from_the_tse_mac, eth_mode_from_the_tse_mac,pll_clk_125,pll_clk_2, pll_clk_25)begin
		if(eth_mode_from_the_tse_mac = '1')then --1000M
			tx_clk_to_the_tse_mac <= pll_clk_125;
			pcie_led_x1 <= '0';
			pcie_led_x4 <= '1';
			pcie_led_x8 <= '1';
		elsif(ena_10_from_the_tse_mac = '1')then -- 10M
			tx_clk_to_the_tse_mac <= pll_clk_2;
			pcie_led_x1 <= '1';
			pcie_led_x4 <= '1';
			pcie_led_x8 <= '0';
		else -- 100M
			tx_clk_to_the_tse_mac <= pll_clk_25;
			pcie_led_x1 <= '1';
			pcie_led_x4 <= '0';
			pcie_led_x8 <= '1';
		end if;
	end process;


	mdio_in_to_the_tse_mac <= enet_mdio;




--	max2_csn <= max2_cs_int or max2_adsc_int;

	flash_cen <= flash_ce and flash_1_ce;
	flash_oen <= flash_oe and flash_1_oe;
	flash_wen <= flash_we and flash_1_we;

	fsm_a(25) <= not flash_1_ce;
	
	flash_resetn <= '1';
	flash_advn <= '0';
	flash_clk <= '0';

	sram_gwn <= '1';
	sram_zz <= '0';


	this_pll_125 : pll_125 port map(clkin_bot_p, pll_clk_125, pll_clk_25, pll_clk_2);
	enet_gtx_clk_ddio_buffer_inst : enet_gtx_clk_ddio_buffer port map(
		not cpu_resetn,
		'1',
		'0',
		tx_clk_to_the_tse_mac,
		enet_gtx_clk
    );
--	enet_gtx_clk <= tx_clk_to_the_tse_mac;

	process(cpu_resetn, clkin_bot_p)begin
		if(cpu_resetn = '0')then
			count <= 0;
		elsif(clkin_bot_p'event and clkin_bot_p = '1')then
			if(count = 65535)then
				count <= count;
			else
				count <= count + 1;
			end if;
		end if;
	end process;
	
	process(cpu_resetn, clkin_bot_p)begin
		if(cpu_resetn = '0')then
		elsif(clkin_bot_p'event and clkin_bot_p = '1')then
			if(count < 1024)then
				reset_int <= '1';
			elsif(count < 65530)then
				reset_int <= '0';
			else
				reset_int <= '1';			
			end if;
		end if;
	end process;
	
	enet_resetn <= reset_int;
	
	sram_clk <= sram_clk_int;
	max2_clk <= sram_clk_int;
	
end rtl;
